@@ -27,54 +27,63 @@
 #'   the computation proceeds.
 #' @param method character.
 #' @param method.args list of arguments to pass to \code{method}.
-#' @param label.x.npc,label.y.npc \code{numeric} with range 0..1 or character.
-#'   Coordinates to be used for positioning the output, expressed in "normalized
-#'   parent coordinates" or character string. If too short they will be
-#'   recycled.
-#' @param label.x,label.y \code{numeric} Coordinates (in data units) to be used
-#'   for absolute positioning of the output. If too short they will be recycled.
+#' @param label.x,label.y \code{numeric} with range 0..1 "normalized parent
+#'   coordinates" (npc units) or character if using \code{geom_text_npc()} or
+#'   \code{geom_label_npc()}. If using \code{geom_text()} or \code{geom_label()}
+#'   numeric in native data units. If too short they will be recycled.
+#' @param hstep,vstep numeric in npc units, the horizontal and vertical step
+#'   used between labels for different groups.
 #'
 #' @section Computed variables: The output of \code{\link[broom]{glance}} is
 #'   returned as is in the \code{data} object. If you do not know what names
 #'   to expect for the variables returned, use \code{broom::glance()} and
 #'   \code{names()} or \code{print()} to find out.
 #'
+#' @section Warning!: The current implementation works only with methods that
+#'   accept a formula as argument and which have a \code{data} paremeter through
+#'   which a data frame can be passed. For example, \code{lm()} should be
+#'   used with the formula interface, as the evaluation of \code{x} and \code{y}
+#'   needs to be delayed until the internal \code{object} of the ggplot is
+#'   available.
+#'
 #' @note The names of the columns in the returned data are consitent with those
 #'   returned by method \code{glance()} from package 'broom', that will
-#'   frequently differ from the name of values returned by the fit or test
-#'   function used.
+#'   frequently differ from the name of values returned by the print methods
+#'   corresponding to fit or test function used. With some methods like
+#'   \code{cor.test()} the data embedded in the \code{"ggplot"} object cannot be
+#'   automatically passed as argument for the \code{data} parameter of the test
+#'   or model fit function.
 #'
 #' @section Warning!: \code{stat_fit_glance} applies the function given by
 #'   \code{method} separately to each group of observations, and factors mapped
 #'   to aesthetics generate a separate group for each factor level. Because of
 #'   this, it is not useful for annotating plots with results from
 #'   \code{t.test()} or ANOVA or ANCOVA. In such cases use the
-#'   \code{stat_fit_tb} statistic which does the model fitting per panel.
+#'   \code{stat_fit_tb()} statistic which does the model fitting per panel.
 #'
 #' @export
 #'
 #' @examples
-#' library(ggplot2)
-#' # Correlation example
-#' x <- c(44.4, 45.9, 41.9, 53.3, 44.7, 44.1, 50.7, 45.2, 60.1)
-#' y <- c( 2.6,  3.1,  2.5,  5.0,  3.6,  4.0,  5.2,  2.8,  3.8)
-#' my.df <- data.frame(x, y)
-#' # estimated correlation is in column "estimate"!
-#' broom::glance(cor.test(x, y))
-#' ggplot(my.df, aes(x, y)) +
+#' # Regression example
+#' my.df <-
+#'   data.frame(X = c(44.4, 45.9, 41.9, 53.3, 44.7, 44.1, 50.7, 45.2, 60.1),
+#'              Y = c( 2.6,  3.1,  2.5,  5.0,  3.6,  4.0,  5.2,  2.8,  3.8))
+#' # We need to check the names of the returned values!
+#' broom::glance(lm(formula = Y ~ X, data = my.df ))
+#' ggplot(my.df, aes(X, Y)) +
 #'   geom_point() +
-#'   stat_fit_glance(geom = "text",
-#'                   method = "cor.test",
-#'                   method.args = list(x = x, y = y, method = "spearman"),
-#'                   aes(label = sprintf('r[s]~"="~%.2f~~italic(P)~"="~%.2f',
-#'                       stat(estimate), stat(p.value))),
+#'   stat_fit_glance(method = "lm",
+#'                   method.args = list(formula = y ~ x),
+#'                   aes(label = sprintf('r^2~"="~%.3f~~italic(P)~"="~%.2f',
+#'                       stat(r.squared), stat(p.value))),
 #'                   parse = TRUE)
 #'
-stat_fit_glance <- function(mapping = NULL, data = NULL, geom = "null",
+stat_fit_glance <- function(mapping = NULL, data = NULL, geom = "text_npc",
                             method = "lm",
                             method.args = list(formula = y ~ x),
-                            label.x.npc = "left", label.y.npc = "top",
-                            label.x = NULL, label.y = NULL,
+                            label.x = "left", label.y = "top",
+                            hstep = 0,
+                            vstep = NULL,
                             position = "identity",
                             na.rm = FALSE, show.legend = FALSE,
                             inherit.aes = TRUE, ...) {
@@ -83,10 +92,9 @@ stat_fit_glance <- function(mapping = NULL, data = NULL, geom = "null",
     position = position, show.legend = show.legend, inherit.aes = inherit.aes,
     params = list(method = method,
                   method.args = method.args,
-                  label.x.npc = label.x.npc,
-                  label.y.npc = label.y.npc,
                   label.x = label.x,
                   label.y = label.y,
+                  npc.used = grepl("_npc", geom),
                   na.rm = na.rm,
                   ...)
   )
@@ -103,11 +111,11 @@ fit_glance_compute_group_fun <- function(data,
                                          scales,
                                          method,
                                          method.args,
-                                         label.x.npc,
-                                         label.y.npc,
                                          label.x,
-                                         label.y) {
-  force(data)
+                                         label.y,
+                                         npc.used) {
+
+  force(data) # needed because it appears only wihtin quote()
 
   if (length(unique(data$x)) < 2) {
     # Not enough data to perform fit
@@ -115,15 +123,15 @@ fit_glance_compute_group_fun <- function(data,
   }
 
   group.idx <- abs(data$group[1])
-  if (length(label.x.npc) >= group.idx) {
-    label.x.npc <- label.x.npc[group.idx]
-  } else if (length(label.x.npc) > 0) {
-    label.x.npc <- label.x.npc[1]
+  if (length(label.x) >= group.idx) {
+    label.x <- label.x[group.idx]
+  } else if (length(label.x) > 0) {
+    label.x <- label.x[1]
   }
-  if (length(label.y.npc) >= group.idx) {
-    label.y.npc <- label.y.npc[group.idx]
-  } else if (length(label.y.npc) > 0) {
-    label.y.npc <- label.y.npc[1]
+  if (length(label.y) >= group.idx) {
+    label.y <- label.y[group.idx]
+  } else if (length(label.y) > 0) {
+    label.y <- label.y[1]
   }
 
   if (length(label.x) >= group.idx) {
@@ -137,67 +145,75 @@ fit_glance_compute_group_fun <- function(data,
     label.y <- label.y[1]
   }
 
-  method.args <- c(method.args, list(data = quote(data)))
   if (is.character(method)) method <- match.fun(method)
+  if (!any(grepl("formula|fixed|random", names(method.args)))) {
+    warning("Only the 'formula' interface of methods is supported.")
+    return(data.frame())
+  }
+  if ("data" %in% names(method.args)) {
+    message("External 'data' passed to method, possibly inconsistent with plot!\n",
+            "These data must be available at the time of printing!!!")
+  } else {
+    method.args <- c(method.args, list(data = quote(data)))
+  }
   mf <- do.call(method, method.args)
   z <- broom::glance(mf)
 
-  if (length(label.x) > 0) {
-    z$x <- label.x
-    z$hjust <- 0.5
-  } else if (length(label.x.npc) > 0) {
-    if (is.numeric(label.x.npc)) {
-      if (any(label.x.npc < 0 | label.x.npc > 1)) {
-        warning("'label.x.npc' argument is numeric but outside range 0..1.")
-      }
-      z$x <- scales$x$dimension()[1] + label.x.npc *
-        diff(scales$x$dimension())
-      z$hjust <- 0.5
-    } else if (is.character(label.x.npc)) {
-      if (label.x.npc == "right") {
-        z$x <- scales$x$dimension()[2]
-        z$hjust <- 1
-      } else if (label.x.npc %in% c("center", "centre", "middle")) {
-        z$x <- mean(scales$x$dimension())
-        z$hjust <- 0.5
-      } else if (label.x.npc == "left") {
-        z$x <- scales$x$dimension()[1]
-        z$hjust <- 0
-      } else {
-        stop("'label.x.npc' argument '", label.x.npc, " unsupported")
-      }
+  n.labels <- nrow(z)
+  if (length(label.x) != n.labels) {
+    if (length(label.x) != 1L) {
+      warning("Length of 'label.x' is different from number of labels")
+    }
+    if (length(label.x) < n.labels) {
+      label.x <- rep(label.x[1], n.labels)
     } else {
-      stop("'label.x.npc' argument is neither numeric nor character")
+      label.x <- label.x[1:n.labels]
+    }
+  }
+  if (length(label.y) != n.labels) {
+    if (length(label.y) != 1L) {
+      warning("Length of 'label.y' is different from number of labels")
+    }
+    if (length(label.y) < n.labels) {
+      label.y <- rep(label.y[1], n.labels)
+    } else {
+      label.y <- label.y[1:n.labels]
     }
   }
 
-  if (length(label.y) > 0) {
-    z$y <- label.y
-    z$vjust <- 0.5
-  } else if (length(label.y.npc) > 0) {
-    if (is.numeric(label.y.npc)) {
-      if (any(label.y.npc < 0 | label.y.npc > 1)) {
-        warning("'label.y.npc' argument is numeric but outside range 0..1.")
-      }
-      z$y <- scales$y$dimension()[1] + label.y.npc *
-        diff(scales$y$dimension())
-      z$vjust <- 1.4 * group.idx - (0.7 * length(group.idx))
-    } else if (is.character(label.y.npc)) {
-      if (label.y.npc == "bottom") {
-        z$y <- scales$y$dimension()[1]
-        z$vjust <- -1.4 * group.idx
-      } else if (label.y.npc %in% c("center", "centre", "middle")) {
-        z$y <- mean(scales$y$dimension())
-        z$vjust <- 1.4 * group.idx - (0.7 * length(group.idx))
-      } else if (label.y.npc == "top") {
-        z$y <- scales$y$dimension()[2]
-        z$vjust <- 1.4 * group.idx
-      } else {
-        stop("'label.y.npc' argument '", label.y.npc, " unsupported")
-      }
-    } else {
-      stop("'label.y.npc' argument is neither numeric nor character")
+  if (npc.used) {
+    margin.npc = 0.05
+    hstep <- 0
+    hsteps <- hstep * (group.idx - 1L)
+    margin.npc = 0.05
+    if (is.character(label.x)) {
+      label.x <- switch(label.x,
+                        right = (1 - margin.npc) - hsteps,
+                        center = 0.5 - hsteps,
+                        centre = 0.5 - hsteps,
+                        middle = 0.5 - hsteps,
+                        left = (0 + margin.npc) +  hsteps)
     }
+    vstep <- 0.075
+    vsteps <- vstep * (group.idx - 1L)
+    if (is.character(label.y)) {
+      label.y <- switch(label.y,
+                        top = (1 - margin.npc) - vsteps,
+                        center = 0.5 - vsteps,
+                        centre = 0.5 - vsteps,
+                        middle = 0.5 - vsteps,
+                        bottom = (0 + margin.npc) + vsteps
+      )
+    }
+    z$npcx <- label.x
+    z$x <- NA_real_
+    z$npcy <- label.y
+    z$y <- NA_real_
+  } else {
+    z$x <- label.x
+    z$npcx <- NA_real_
+    z$y <- label.y
+    z$npcy <- NA_real_
   }
 
   z
@@ -211,11 +227,13 @@ StatFitGlance <-
   ggplot2::ggproto("StatFitGlance", ggplot2::Stat,
                    compute_group = fit_glance_compute_group_fun,
                    default_aes =
-                     ggplot2::aes(hjust = ..hjust.., vjust = ..vjust..),
+                     ggplot2::aes(npcx = stat(npcx),
+                                  npcy = stat(npcy),
+                                  hjust = "inward",
+                                  vjust = "inward"),
                    required_aes = c("x", "y")
   )
 
-
 # broom::augment ----------------------------------------------------------
 
 #' @title Augment data with fitted values and statistics
@@ -352,7 +370,10 @@ StatFitAugment <-
 #' @title One row data frame with fitted parameter estimates
 #'
 #' @description \code{stat_fit_tidy} fits a model and returns a "tidy" version
-#'   of the model's summary, using package 'broom'.
+#'   of the model's summary, using package 'broom'. To add the summary in
+#'   tabular form use \code{\link{stat_fit_tb}}. When using
+#'   \code{stat_fit_tidy()} you will most likely want to change the default
+#'   mapping for label.
 #'
 #' @param mapping The aesthetic mapping, usually constructed with
 #'   \code{\link[ggplot2]{aes}} or \code{\link[ggplot2]{aes_}}. Only needs
@@ -376,11 +397,11 @@ StatFitAugment <-
 #'   before the computation proceeds.
 #' @param method character.
 #' @param method.args list of arguments to pass to \code{method}.
-#' @param label.x.npc,label.y.npc \code{numeric} with range 0..1 or character.
+#' @param label.x,label.y \code{numeric} with range 0..1 or character.
 #'   Coordinates to be used for positioning the output, expressed in "normalized
 #'   parent coordinates" or character string. If too short they will be recycled.
-#' @param label.x,label.y \code{numeric} Coordinates (in data units) to be used
-#'   for absolute positioning of the output. If too short they will be recycled.
+#' @param hstep,vstep numeric in npc units, the horizontal and vertical step
+#'   used between labels for different groups.
 #'
 #' @section Computed variables: The output of \code{\link[broom]{tidy}} is
 #'   returned after reshaping it into a single row. Grouping is respected, and
@@ -389,23 +410,43 @@ StatFitAugment <-
 #'
 #' @export
 #'
-stat_fit_tidy <- function(mapping = NULL, data = NULL, geom = "null",
-                            method = "lm",
-                            method.args = list(formula = y ~ x),
-                            label.x.npc = "left", label.y.npc = "top",
-                            label.x = NULL, label.y = NULL,
-                            position = "identity",
-                            na.rm = FALSE, show.legend = FALSE,
-                            inherit.aes = TRUE, ...) {
+#' @examples
+#'
+#' # Regression example
+#' my.df <-
+#'   data.frame(X = c(44.4, 45.9, 41.9, 53.3, 44.7, 44.1, 50.7, 45.2, 60.1),
+#'              Y = c( 2.6,  3.1,  2.5,  5.0,  3.6,  4.0,  5.2,  2.8,  3.8))
+#'
+#' ggplot(my.df, aes(X, Y)) +
+#'   geom_point() +
+#'   stat_fit_tidy(method = "lm",
+#'                 method.args = list(formula = y ~ x),
+#'                 mapping = aes(label = sprintf("Slope = %.3g\np-value = %.3g",
+#'                                               stat(x_estimate), stat(x_p.value))))
+#'
+stat_fit_tidy <- function(mapping = NULL, data = NULL, geom = "text_npc",
+                          method = "lm",
+                          method.args = list(formula = y ~ x),
+                          label.x = "left", label.y = "top",
+                          hstep = 0,
+                          vstep = NULL,
+                          position = "identity",
+                          na.rm = FALSE, show.legend = FALSE,
+                          inherit.aes = TRUE, ...) {
   ggplot2::layer(
     stat = StatFitTidy, data = data, mapping = mapping, geom = geom,
     position = position, show.legend = show.legend, inherit.aes = inherit.aes,
     params = list(method = method,
                   method.args = method.args,
-                  label.x.npc = label.x.npc,
-                  label.y.npc = label.y.npc,
                   label.x = label.x,
                   label.y = label.y,
+                  hstep = hstep,
+                  vstep = ifelse(is.null(vstep),
+                                 ifelse(grepl("label", geom),
+                                        0.125,
+                                        0.075),
+                                 vstep),
+                  npc.used = grepl("_npc", geom),
                   na.rm = na.rm,
                   ...)
   )
@@ -419,13 +460,14 @@ stat_fit_tidy <- function(mapping = NULL, data = NULL, geom = "null",
 #' @usage NULL
 #'
 fit_tidy_compute_group_fun <- function(data,
-                                         scales,
-                                         method,
-                                         method.args,
-                                         label.x.npc,
-                                         label.y.npc,
-                                         label.x,
-                                         label.y) {
+                                       scales,
+                                       method,
+                                       method.args,
+                                       label.x,
+                                       label.y,
+                                       hstep,
+                                       vstep,
+                                       npc.used) {
   force(data)
   if (length(unique(data$x)) < 2) {
     # Not enough data to perform fit
@@ -433,15 +475,15 @@ fit_tidy_compute_group_fun <- function(data,
   }
 
   group.idx <- abs(data$group[1])
-  if (length(label.x.npc) >= group.idx) {
-    label.x.npc <- label.x.npc[group.idx]
-  } else if (length(label.x.npc) > 0) {
-    label.x.npc <- label.x.npc[1]
+  if (length(label.x) >= group.idx) {
+    label.x <- label.x[group.idx]
+  } else if (length(label.x) > 0) {
+    label.x <- label.x[1]
   }
-  if (length(label.y.npc) >= group.idx) {
-    label.y.npc <- label.y.npc[group.idx]
-  } else if (length(label.y.npc) > 0) {
-    label.y.npc <- label.y.npc[1]
+  if (length(label.y) >= group.idx) {
+    label.y <- label.y[group.idx]
+  } else if (length(label.y) > 0) {
+    label.y <- label.y[1]
   }
 
   if (length(label.x) >= group.idx) {
@@ -476,64 +518,37 @@ fit_tidy_compute_group_fun <- function(data,
     z <- cbind(z, z.p.value)
   }
 
-  if (length(label.x) > 0) {
+  if (npc.used) {
+    margin.npc = 0.05
+    hsteps <- hstep * (group.idx - 1L)
+    if (is.character(label.x)) {
+      label.x <- switch(label.x,
+                        right = (1 - margin.npc) - hsteps,
+                        center = 0.5 - hsteps,
+                        centre = 0.5 - hsteps,
+                        middle = 0.5 - hsteps,
+                        left = (0 + margin.npc) +  hsteps)
+    }
+    vsteps <- vstep * (group.idx - 1L)
+    if (is.character(label.y)) {
+      label.y <- switch(label.y,
+                        top = (1 - margin.npc) - vsteps,
+                        center = 0.5 - vsteps,
+                        centre = 0.5 - vsteps,
+                        middle = 0.5 - vsteps,
+                        bottom = (0 + margin.npc) + vsteps
+      )
+    }
+    z$npcx <- label.x
+    z$x <- NA_real_
+    z$npcy <- label.y
+    z$y <- NA_real_
+  } else {
     z$x <- label.x
-    z$hjust <- 0.5
-  } else if (length(label.x.npc) > 0) {
-    if (is.numeric(label.x.npc)) {
-      if (any(label.x.npc < 0 | label.x.npc > 1)) {
-        warning("'label.x.npc' argument is numeric but outside range 0..1.")
-      }
-      z$x <- scales$x$dimension()[1] + label.x.npc *
-        diff(scales$x$dimension())
-      z$hjust <- 0.5
-    } else if (is.character(label.x.npc)) {
-      if (label.x.npc == "right") {
-        z$x <- scales$x$dimension()[2]
-        z$hjust <- 1
-      } else if (label.x.npc %in% c("center", "centre", "middle")) {
-        z$x <- mean(scales$x$dimension())
-        z$hjust <- 0.5
-      } else if (label.x.npc == "left") {
-        z$x <- scales$x$dimension()[1]
-        z$hjust <- 0
-      } else {
-        stop("'label.x.npc' argument '", label.x.npc, " unsupported")
-      }
-    } else {
-      stop("'label.x.npc' argument is neither numeric nor character")
-    }
-  }
-
-  if (length(label.y) > 0) {
+    z$npcx <- NA_real_
     z$y <- label.y
-    z$vjust <- 0.5
-  } else if (length(label.y.npc) > 0) {
-    if (is.numeric(label.y.npc)) {
-      if (any(label.y.npc < 0 | label.y.npc > 1)) {
-        warning("'label.y.npc' argument is numeric but outside range 0..1.")
-      }
-      z$y <- scales$y$dimension()[1] + label.y.npc *
-        diff(scales$y$dimension())
-      z$vjust <- 1.4 * group.idx - (0.7 * length(group.idx))
-    } else if (is.character(label.y.npc)) {
-      if (label.y.npc == "bottom") {
-        z$y <- scales$y$dimension()[1]
-        z$vjust <- -1.4 * group.idx
-      } else if (label.y.npc %in% c("center", "centre", "middle")) {
-        z$y <- mean(scales$y$dimension())
-        z$vjust <- 1.4 * group.idx - (0.7 * length(group.idx))
-      } else if (label.y.npc == "top") {
-        z$y <- scales$y$dimension()[2]
-        z$vjust <- 1.4 * group.idx
-      } else {
-        stop("'label.y.npc' argument '", label.y.npc, " unsupported")
-      }
-    } else {
-      stop("'label.y.npc' argument is neither numeric nor character")
-    }
+    z$npcy <- NA_real_
   }
-
   z
 }
 
@@ -545,6 +560,9 @@ StatFitTidy <-
   ggplot2::ggproto("StatFitTidy", ggplot2::Stat,
                    compute_group = fit_tidy_compute_group_fun,
                    default_aes =
-                     ggplot2::aes(hjust = ..hjust.., vjust = ..vjust..),
+                     ggplot2::aes(npcx = stat(npcx),
+                                  npcy = stat(npcy),
+                                  hjust = "inward",
+                                  vjust = "inward"),
                    required_aes = c("x", "y")
   )
